# -*- coding: utf-8 -*-
import json
from urllib.request import urlparse
import scrapy
from scrapy.utils.project import get_project_settings
from scrapy_splash import SplashRequest
from w3lib.http import basic_auth_header
from pkgutil import get_data

from .base import BaseSpider
from ..items import ExaItem

api_key = get_project_settings().get('CRAWLERA_APIKEY')


class CbSpider(BaseSpider):
    name = "cb"
    allowed_domains = ["www.crunchbase.com"]
    handle_httpstatus_list = [470]
    crawlera_enabled = True
    crawlera_apikey = api_key
    co = 0

    def __init__(self, *args, **kwargs):
        self.LUA_SOURCE = get_data(
            'exa', 'scripts/crawlera.lua'
        ).decode('utf-8')
        super(CbSpider, self).__init__(*args, **kwargs)

    def start_requests(self):
        for s, i in enumerate(self.companies(self.name)):
            try:
                yield scrapy.Request(url=i.url,
                                     callback=self.parse,
                                     meta={'company': i, 'post_id': 0, 'cookiejar': s})
            except:
                pass

    def parse(self, response):
        body = {
            "field_ids": [
                "activity_properties",
                "entity_def_id",
                "identifier",
                "activity_date",
                "activity_entities"
            ],
            "order": [],
            "query": [
                {
                    "type": "predicate",
                    "field_id": "activity_entities",
                    "operator_id": "includes",
                    "values": [
                        "f93d65c7-11da-f085-0bdd-d54510f77a41"
                    ]
                }
            ],
            "limit": 100
        }
        uuid = '"uuid":"'
        page = response.body.decode('utf8')
        s_uuid = page.find(uuid)
        uuid = page[s_uuid + len(uuid):page.find('"', s_uuid + len(uuid))]
        body['query'][0]['values'][0] = uuid
        yield scrapy.Request(url='https://www.crunchbase.com/v4/data/searches/activities',
                             method='POST',
                             body=json.dumps(body),
                             headers={'x-requested-with': 'XMLHttpRequest', 'content-type': 'application/json'},
                             callback=self.parse_news,
                             meta={'cookiejar': response.meta['cookiejar'],
                                   'company': response.meta['company'],
                                   'post_id': response.meta['post_id']})

    def parse_news(self, response):
        body = json.loads(response.body.decode('utf8'))
        for i in body['entities']:
            prop = i['properties']
            if prop['entity_def_id'] == 'press_reference':
                item = ExaItem()
                item['date'] = self.format_date(prop['activity_date'])
                item['title'] = prop['activity_properties']['title']
                item['url'] = prop['activity_properties']['url']['value']
                publisher = prop['activity_properties']['publisher']
                item.update(self.get_common_items(response.meta['company']))
                item['media_id'] = self._get_media((publisher, item['url']))
                item['description'] = None
                item['post_id'] = response.meta['post_id']
                item['tags'] = None
                if self.pipeline.check_url(item['url']) and self.fresh:
                        break
                yield item

    def _get_media(self, site):
        media_name, media_url = site
        clean = lambda x: x[4:] if x.startswith('www.') else x
        media_url = clean(urlparse(media_url).netloc)
        query = "select * from wp_esi_media where name like '%{}%' or url like '%{}%'".format(media_name, media_url)
        media = self.pipeline.db.select(query)
        if len(media) == 0:
            media = self.pipeline.db.insert("INSERT INTO wp_esi_media (name, url) VALUES(%s, %s)",
                                            (media_name, media_url))
        else:
            media = media[0][0]
        return media
